@@ -10,7 +10,6 @@
 #include "SparseArray_aperm.h"
 #include "SparseArray_subsetting.h"
 #include "SparseArray_subassignment.h"
-#include "SparseArray_subassignment_OLD.h"
 #include "SparseArray_abind.h"
 #include "SparseArray_summarization.h"
 #include "SparseArray_Arith_methods.h"
@@ -75,15 +74,11 @@ static const R_CallMethodDef callMethods[] = {
 
 /* SparseArray_subassignment.c */
 	CALLMETHOD_DEF(C_subassign_SVT_by_Lindex, 6),
-	CALLMETHOD_DEF(C_subassign_SVT_by_Mindex, 5),
+	CALLMETHOD_DEF(C_subassign_SVT_by_Mindex, 6),
 	CALLMETHOD_DEF(C_subassign_SVT_with_short_Rvector, 5),
 	CALLMETHOD_DEF(C_subassign_SVT_with_Rarray, 6),
 	CALLMETHOD_DEF(C_subassign_SVT_with_SVT, 7),
 
-/* SparseArray_subassignment_OLD.c */
-	CALLMETHOD_DEF(C_subassign_SVT_by_Lindex_OLD, 5),
-	CALLMETHOD_DEF(C_subassign_SVT_by_Mindex_OLD, 5),
-
 /* SparseArray_abind.c */
 	CALLMETHOD_DEF(C_abind_SVT_SparseArray_objects, 4),
 
@@ -147,6 +142,9 @@ void R_init_SparseArray(DllInfo *info)
 {
 	R_registerRoutines(info, NULL, callMethods, NULL, NULL);
 	R_useDynamicSymbols(info, 0);
+
+	/* 'character1' will remain PROTECT'ed for the entire R session. */
+	character1 = PROTECT(mkChar("1"));  /* CHARSXP */
 	intNA = NA_INTEGER;
 	doubleNA = RcomplexNA.r = RcomplexNA.i = NA_REAL;
 	return;
