@@ -3,8 +3,6 @@
  ****************************************************************************/
 #include "SVT_SparseArray_class.h"
 
-#include "S4Vectors_interface.h"  /* for sort_ints() */
-
 #include "argcheck_utils.h"
 #include "Rvector_utils.h"
 #include "coerceVector2.h"  /* for _CoercionWarning() */
@@ -14,18 +12,17 @@
 #include <string.h>  /* for strcmp() and memcpy() */
 
 
-static void copy_nzvals_elts_to_Rsubvec(SEXP nzvals,
-		SEXP out, int out_offset, int nelt)
+static void copy_nzvals_to_Rvector_block(SEXP nzvals,
+		SEXP Rvector, int block_offset, int block_len)
 {
 	if (nzvals == R_NilValue) {
 		/* lacunar leaf */
-		_set_Rsubvec_elts_to_one(out, (R_xlen_t) out_offset,
-					 (R_xlen_t) nelt);
+		_fill_Rvector_block_with_ones(Rvector,
+			(R_xlen_t) block_offset, (R_xlen_t) block_len);
 	} else {
 		/* regular leaf */
-		_copy_Rvector_elts(nzvals, 0,
-				   out, (R_xlen_t) out_offset,
-				   (R_xlen_t) nelt);
+		_copy_Rvector_elts(nzvals, 0, Rvector,
+			(R_xlen_t) block_offset, (R_xlen_t) block_len);
 	}
 	return;
 }
@@ -355,9 +352,9 @@ static int REC_extract_nzcoo_and_nzvals_from_SVT(SEXP SVT,
 	int leaf_nzcount = unzip_leaf(SVT, &leaf_nzvals, &leaf_nzoffs);
 
 	if (out_nzvals != R_NilValue)
-		copy_nzvals_elts_to_Rsubvec(leaf_nzvals,
-					    out_nzvals, *nzvals_offset,
-					    leaf_nzcount);
+		copy_nzvals_to_Rvector_block(leaf_nzvals,
+					     out_nzvals, *nzvals_offset,
+					     leaf_nzcount);
 
 	for (int k = 0; k < leaf_nzcount; k++) {
 		rowbuf[0] = INTEGER(leaf_nzoffs)[k] + 1;
@@ -501,7 +498,7 @@ SEXP C_from_SVT_SparseArray_to_Rarray(SEXP x_dim, SEXP x_dimnames,
  */
 
 /* Recursive. */
-static SEXP REC_build_SVT_from_Rsubarray(
+static SEXP REC_build_SVT_from_Rsubarr(
 		SEXP Rarray, R_xlen_t arr_offset, R_xlen_t subarr_len,
 		const int *dim, int ndim,
 		SEXPTYPE ans_Rtype, int ans_na_background,
@@ -511,15 +508,15 @@ static SEXP REC_build_SVT_from_Rsubarray(
 		/* Sanity check (should never fail). */
 		if (dim[0] != subarr_len)
 			error("SparseArray internal error in "
-			      "REC_build_SVT_from_Rsubarray():\n"
+			      "REC_build_SVT_from_Rsubarr():\n"
 			      "    dim[0] != subarr_len");
 		SEXP ans;
 		if (ans_na_background) {
-			ans = _make_naleaf_from_Rsubvec(
+			ans = _make_naleaf_from_Rvector_block(
 					Rarray, arr_offset, dim[0],
 					offs_buf, 1);
 		} else {
-			ans = _make_leaf_from_Rsubvec(
+			ans = _make_leaf_from_Rvector_block(
 					Rarray, arr_offset, dim[0],
 					offs_buf, 1);
 		}
@@ -540,7 +537,7 @@ static SEXP REC_build_SVT_from_Rsubarray(
 	SEXP ans = PROTECT(NEW_LIST(SVT_len));
 	int is_empty = 1;
 	for (int i = 0; i < SVT_len; i++) {
-		SEXP ans_elt = REC_build_SVT_from_Rsubarray(
+		SEXP ans_elt = REC_build_SVT_from_Rsubarr(
 					Rarray, arr_offset, subarr_len,
 					dim, ndim - 1,
 					ans_Rtype, ans_na_background,
@@ -575,7 +572,7 @@ SEXP C_build_SVT_from_Rarray(SEXP x, SEXP ans_type, SEXP ans_na_background)
 	int x_ndim = LENGTH(x_dim);
 	int *offs_buf = (int *) R_alloc(INTEGER(x_dim)[0], sizeof(int));
 	int warn = 0;
-	SEXP ans = REC_build_SVT_from_Rsubarray(x, 0, x_len,
+	SEXP ans = REC_build_SVT_from_Rsubarr(x, 0, x_len,
 				INTEGER(x_dim), x_ndim,
 				ans_Rtype, ans_has_NAbg,
 				&warn, offs_buf);
@@ -612,7 +609,7 @@ static int dump_leaf_to_ix(SEXP leaf,
 		return nzcount;
 
 	/* Copy 'nzvals' to 'slotx'. */
-	copy_nzvals_elts_to_Rsubvec(nzvals, slotx, ix_offset, nzcount);
+	copy_nzvals_to_Rvector_block(nzvals, slotx, ix_offset, nzcount);
 	return nzcount;
 }
 
@@ -721,8 +718,8 @@ static SEXP build_leaf_from_CsparseMatrix_col(const int *sloti, SEXP slotx,
 		SEXPTYPE ans_Rtype, int *warn, int *nzoffs_buf)
 {
 	/* 'slotx' can contain zeros. See above. */
-	SEXP ans = _make_leaf_from_Rsubvec(slotx, ix_offset, col_nzcount,
-					   nzoffs_buf, 1);
+	SEXP ans = _make_leaf_from_Rvector_block(slotx, ix_offset, col_nzcount,
+						 nzoffs_buf, 1);
 	if (ans == R_NilValue)
 		return ans;
 
