@@ -49,8 +49,9 @@ void _expand_leaf(SEXP leaf, SEXP out_Rvector, R_xlen_t out_offset)
 	SEXP nzvals, nzoffs;
 	unzip_leaf(leaf, &nzvals, &nzoffs);  /* ignore returned nzcount */
 	if (nzvals == R_NilValue) {  /* lacunar leaf */
-		_set_selected_Rsubvec_elts_to_one(out_Rvector, out_offset,
-					INTEGER(nzoffs), LENGTH(nzoffs));
+		_fill_Rvector_subset_with_ones(out_Rvector,
+					INTEGER(nzoffs), LENGTH(nzoffs),
+					out_offset);
 
 	} else {  /* standard leaf */
 		_copy_Rvector_elts_to_offsets(nzvals, INTEGER(nzoffs),
@@ -64,8 +65,8 @@ void _expand_leaf(SEXP leaf, SEXP out_Rvector, R_xlen_t out_offset)
  * _make_lacunar_leaf()
  * _make_leaf_with_single_shared_nzval()
  * _make_leaf_from_two_arrays()
- * _make_leaf_from_Rsubvec()
- * _make_naleaf_from_Rsubvec()
+ * _make_leaf_from_Rvector_block()
+ * _make_naleaf_from_Rvector_block()
  */
 
 SEXP _make_lacunar_leaf(SEXP nzoffs)
@@ -81,56 +82,71 @@ SEXP _make_leaf_with_single_shared_nzval(SEXPTYPE Rtype,
 	if (_all_elts_equal_one(Rtype, shared_nzval, 1))
 		return _make_lacunar_leaf(nzoffs);
 	SEXP nzvals = PROTECT(allocVector(Rtype, LENGTH(nzoffs)));
-	_set_Rvector_elts_to_val(nzvals, shared_nzval);
+	_fill_Rvector_with_val(nzvals, shared_nzval);
 	SEXP ans = zip_leaf(nzvals, nzoffs, 0);
 	UNPROTECT(1);
 	return ans;
 }
 
-/* Does NOT work if 'Rtype' is STRSXP or VECSXP.
-   Each of 'nzvals_p' and 'nzoffs_p' must be a pointer to an array of length
-   'nzcount'. 'nzvals_p' is **trusted** to not contain any zeros. This is NOT
-   checked! The returned leaf can be lacunar. */
+/* Each of 'nzvals_p' and 'nzoffs_p' must be a pointer to an array of
+   length 'nzcount'. 'nzvals_p' is **trusted** to not contain any zeros (this
+   is NOT checked).
+   The returned leaf can be lacunar. */
 SEXP _make_leaf_from_two_arrays(SEXPTYPE Rtype,
 		const void *nzvals_p, const int *nzoffs_p, int nzcount)
 {
 	if (nzcount == 0)
 		return R_NilValue;
-
-	size_t Rtype_size = _get_Rtype_size(Rtype);
-	if (Rtype_size == 0)
-		error("SparseArray internal error in "
-		      "_make_leaf_from_two_arrays():\n"
-		      "    type \"%s\" is not supported", type2char(Rtype));
-
 	SEXP ans_nzoffs = PROTECT(NEW_INTEGER(nzcount));
 	memcpy(INTEGER(ans_nzoffs), nzoffs_p, sizeof(int) * nzcount);
-
-	int all_ones = _all_elts_equal_one(Rtype, nzvals_p, nzcount);
-	if (all_ones) {
-		SEXP ans = _make_lacunar_leaf(ans_nzoffs);
-		UNPROTECT(1);
-		return ans;
+	SEXP ans_nzvals;
+	if (Rtype == STRSXP) {
+		/* Lacunar leaves of Rtype STRSXP are not supported yet
+		   (they will be in the future). */
+		ans_nzvals = PROTECT(NEW_CHARACTER(nzcount));
+		for (int k = 0; k < nzcount; k++) {
+			SEXP nzval = STRING_ELT((SEXP) nzvals_p, k);
+			SET_STRING_ELT(ans_nzvals, k, nzval);
+		}
+	} else if (Rtype == VECSXP) {
+		/* Lacunar leaves of Rtype VECSXP are not supported. */
+		ans_nzvals = PROTECT(NEW_LIST(nzcount));
+		for (int k = 0; k < nzcount; k++) {
+			SEXP nzval = VECTOR_ELT((SEXP) nzvals_p, k);
+			SET_VECTOR_ELT(ans_nzvals, k, nzval);
+		}
+	} else {
+		size_t Rtype_size = _get_Rtype_size(Rtype);
+		if (Rtype_size == 0)
+			error("SparseArray internal error in "
+			      "_make_leaf_from_two_arrays():\n    type "
+			      "\"%s\" is not supported", type2char(Rtype));
+		int all_ones = _all_elts_equal_one(Rtype, nzvals_p, nzcount);
+		if (all_ones) {
+			SEXP ans = _make_lacunar_leaf(ans_nzoffs);
+			UNPROTECT(1);
+			return ans;
+		}
+		ans_nzvals = PROTECT(allocVector(Rtype, nzcount));
+		memcpy(DATAPTR(ans_nzvals), nzvals_p, Rtype_size * nzcount);
 	}
-	SEXP ans_nzvals = PROTECT(allocVector(Rtype, nzcount));
-	memcpy(DATAPTR(ans_nzvals), nzvals_p, Rtype_size * nzcount);
 	SEXP ans = zip_leaf(ans_nzvals, ans_nzoffs, 0);
 	UNPROTECT(2);
 	return ans;
 }
 
-static SEXP make_leaf_from_selected_Rsubvec_elts(
-		SEXP Rvector, R_xlen_t subvec_offset, int subvec_len,
-		const int *selection, int n, int avoid_copy_if_all_selected)
+static SEXP make_leaf_from_Rvector_subset(SEXP Rvector,
+		const int *selection, int selection_len,
+		R_xlen_t selection_offset, int avoid_copy_if_all_selected)
 {
-	if (n == 0)
+	if (selection_len == 0)
 		return R_NilValue;
 
-	SEXP ans_nzoffs = PROTECT(NEW_INTEGER(n));
-	memcpy(INTEGER(ans_nzoffs), selection, sizeof(int) * n);
+	SEXP ans_nzoffs = PROTECT(NEW_INTEGER(selection_len));
+	memcpy(INTEGER(ans_nzoffs), selection, sizeof(int) * selection_len);
 
-	int all_ones = _all_selected_Rsubvec_elts_equal_one(Rvector,
-					     subvec_offset, selection, n);
+	int all_ones = _Rvector_subset_is_filled_with_ones(Rvector,
+				selection, selection_len, selection_offset);
 	if (all_ones) {
 		SEXP ans = _make_lacunar_leaf(ans_nzoffs);
 		UNPROTECT(1);
@@ -138,7 +154,7 @@ static SEXP make_leaf_from_selected_Rsubvec_elts(
 	}
 
 	if (avoid_copy_if_all_selected &&
-	    subvec_offset == 0 && n == XLENGTH(Rvector) &&
+	    selection_offset == 0 && selection_len == XLENGTH(Rvector) &&
 	    ATTRIB(Rvector) == R_NilValue)
 	{
 		/* The full 'Rvector' is selected so can be reused as-is
@@ -149,7 +165,7 @@ static SEXP make_leaf_from_selected_Rsubvec_elts(
 	}
 
 	SEXP ans_nzvals = PROTECT(
-		_subset_Rsubvec(Rvector, subvec_offset, selection, n)
+	  _subset_Rvector(Rvector, selection, selection_len, selection_offset)
 	);
 	SEXP ans = zip_leaf(ans_nzvals, ans_nzoffs, 0);
 	UNPROTECT(2);
@@ -157,34 +173,34 @@ static SEXP make_leaf_from_selected_Rsubvec_elts(
 }
 
 
-/* 'selection_buf' must be of length 'subvec_len' (at least).
+/* 'selection_buf' must be of length 'block_len' (at least).
    The returned leaf can be lacunar. */
-SEXP _make_leaf_from_Rsubvec(
-		SEXP Rvector, R_xlen_t subvec_offset, int subvec_len,
+SEXP _make_leaf_from_Rvector_block(
+		SEXP Rvector, R_xlen_t block_offset, int block_len,
 		int *selection_buf, int avoid_copy_if_all_nonzeros)
 {
-	/* 'n' will always be >= 0 and <= subvec_len. */
-	int n = _collect_offsets_of_nonzero_Rsubvec_elts(
-				Rvector, subvec_offset, subvec_len,
+	/* 'selection_len' will always be >= 0 and <= block_len. */
+	int selection_len = _collect_offsets_of_nonzero_elts_in_Rvector_block(
+				Rvector, block_offset, block_len,
 				selection_buf);
-	return make_leaf_from_selected_Rsubvec_elts(
-				Rvector, subvec_offset, subvec_len,
-				selection_buf, n, avoid_copy_if_all_nonzeros);
+	return make_leaf_from_Rvector_subset(Rvector,
+				selection_buf, selection_len, block_offset,
+				avoid_copy_if_all_nonzeros);
 }
 
-/* 'selection_buf' must be of length 'subvec_len' (at least).
+/* 'selection_buf' must be of length 'block_len' (at least).
    The returned leaf can be lacunar. */
-SEXP _make_naleaf_from_Rsubvec(
-		SEXP Rvector, R_xlen_t subvec_offset, int subvec_len,
+SEXP _make_naleaf_from_Rvector_block(
+		SEXP Rvector, R_xlen_t block_offset, int block_len,
 		int *selection_buf, int avoid_copy_if_all_nonNAs)
 {
-	/* 'n' will always be >= 0 and <= subvec_len. */
-	int n = _collect_offsets_of_nonNA_Rsubvec_elts(
-				Rvector, subvec_offset, subvec_len,
+	/* 'selection_len' will always be >= 0 and <= block_len. */
+	int selection_len = _collect_offsets_of_nonNA_elts_in_Rvector_block(
+				Rvector, block_offset, block_len,
 				selection_buf);
-	return make_leaf_from_selected_Rsubvec_elts(
-				Rvector, subvec_offset, subvec_len,
-				selection_buf, n, avoid_copy_if_all_nonNAs);
+	return make_leaf_from_Rvector_subset(Rvector,
+				selection_buf, selection_len, block_offset,
+				avoid_copy_if_all_nonNAs);
 }
 
 
@@ -201,8 +217,7 @@ SEXP _make_naleaf_from_Rsubvec(
 int _INPLACE_turn_into_lacunar_leaf_if_all_ones(SEXP leaf)
 {
 	SEXP nzvals = get_leaf_nzvals(leaf);
-	int nzcount = LENGTH(nzvals);
-	int all_ones = _all_Rsubvec_elts_equal_one(nzvals, 0, nzcount);
+	int all_ones = _Rvector_is_filled_with_ones(nzvals);
 	if (all_ones)
 		replace_leaf_nzvals(leaf, R_NilValue);
 	return all_ones;
@@ -235,18 +250,18 @@ static int INPLACE_extract_selection_from_leaf(SEXP leaf,
 		return 1;  /* no-op */
 
 	/* Shrink 'nzoffs'. */
-	SEXP new_nzoffs = PROTECT(_subset_Rsubvec(nzoffs, 0, selection, n));
+	SEXP new_nzoffs = PROTECT(_subset_Rvector(nzoffs, selection, n, 0));
 	replace_leaf_nzoffs(leaf, new_nzoffs);
 	UNPROTECT(1);
 
 	/* Shrink 'nzvals'. */
-	int all_ones = _all_selected_Rsubvec_elts_equal_one(nzvals, 0,
-							    selection, n);
+	int all_ones = _Rvector_subset_is_filled_with_ones(nzvals,
+							   selection, n, 0);
 	if (all_ones) {
 		replace_leaf_nzvals(leaf, R_NilValue);
 		return 2;
 	}
-	SEXP new_nzvals = PROTECT(_subset_Rsubvec(nzvals, 0, selection, n));
+	SEXP new_nzvals = PROTECT(_subset_Rvector(nzvals, selection, n, 0));
 	replace_leaf_nzvals(leaf, new_nzvals);
 	UNPROTECT(1);
 	return 3;
@@ -259,7 +274,7 @@ int _INPLACE_remove_zeros_from_leaf(SEXP leaf, int *selection_buf)
 	SEXP nzvals, nzoffs;
 	int nzcount = unzip_leaf(leaf, &nzvals, &nzoffs);
 	/* 'new_nzcount' will always be >= 0 and <= nzcount. */
-	int new_nzcount = _collect_offsets_of_nonzero_Rsubvec_elts(
+	int new_nzcount = _collect_offsets_of_nonzero_elts_in_Rvector_block(
 					nzvals, 0, nzcount, selection_buf);
 	return INPLACE_extract_selection_from_leaf(leaf,
 					selection_buf, new_nzcount);
@@ -272,7 +287,7 @@ int _INPLACE_remove_NAs_from_leaf(SEXP leaf, int *selection_buf)
 	SEXP nzvals, nzoffs;
 	int nzcount = unzip_leaf(leaf, &nzvals, &nzoffs);
 	/* 'new_nzcount' will always be >= 0 and <= nzcount. */
-	int new_nzcount = _collect_offsets_of_nonNA_Rsubvec_elts(
+	int new_nzcount = _collect_offsets_of_nonNA_elts_in_Rvector_block(
 					nzvals, 0, nzcount, selection_buf);
 	return INPLACE_extract_selection_from_leaf(leaf,
 					selection_buf, new_nzcount);
@@ -313,7 +328,7 @@ void _INPLACE_order_leaf_by_nzoff(SEXP leaf, int *order_buf,
 
 	/* regular leaf */
 	SEXP new_nzvals = PROTECT(allocVector(TYPEOF(nzvals), nzcount));
-	_copy_selected_Rsubvec_elts(nzvals, 0, order_buf, new_nzvals);
+	_copy_Rvector_subset(nzvals, order_buf, 0, new_nzvals);
 	replace_leaf_nzvals(leaf, new_nzvals);
 	UNPROTECT(1);
 	return;
@@ -330,7 +345,7 @@ void _INPLACE_order_leaf_by_nzoff(SEXP leaf, int *order_buf,
 
 static SEXP coerce_lacunar_leaf(SEXP leaf, SEXPTYPE new_Rtype)
 {
-	if (new_Rtype != STRSXP && new_Rtype != VECSXP)
+	if (!IS_STRSXP_OR_VECSXP(new_Rtype))
 		return leaf;  /* no-op */
 	error("SparseArray internal error in coerce_lacunar_leaf():\n"
 	      "    coercing a lacunar leaf to \"character\" or \"list\" "
@@ -390,159 +405,88 @@ SEXP _coerce_naleaf(SEXP leaf, SEXPTYPE new_Rtype, int *warn,
 
 
 /****************************************************************************
- * _subassign_leaf_with_vector()
+ * _subassign_leaf_with_Rvector_block()
+ * _subassign_leaf_with_Rvector_subset()
+ * _subassign_leaf_with_Rvector_xsubset()
  */
 
-/* Can be used on a NULL or lacunar leaf. */
-SEXP _subassign_leaf_with_vector(SEXP leaf, SEXP offs,
-				 const void *vals, int n,
-				 SparseVec *buf_sv)
+/* Can be used on a NULL or lacunar leaf.
+   'offs' must be NULL or an array of 'n' offsets (non-negative integers)
+   that are strictly sorted (in ascending order). The last offset in the
+   array must be < 'buf_sv->len'. */
+SEXP _subassign_leaf_with_Rvector_block(SEXP leaf, SEXP offs, int n,
+		SEXP Rvector, R_xlen_t block_offset, SparseVec *buf_sv)
 {
-	const int *offs0 = NULL;
-	if (offs != R_NilValue) {
+	const int *offs0;
+	if (offs == R_NilValue) {
+		if (n != buf_sv->len)
+			error("SparseArray internal error in "
+			      "_subassign_leaf_with_Rvector_block():\n"
+			      "    n != buf_sv->len");
+		offs0 = NULL;
+	} else {
 		if (n != LENGTH(offs))
 			error("SparseArray internal error in "
-			      "_subassign_leaf_with_vector():\n"
-			      "    number of vals != number of offsets");
+			      "_subassign_leaf_with_Rvector_block():\n"
+			      "    n != LENGTH(offs)");
 		offs0 = INTEGER(offs);
 	}
-	if (leaf == R_NilValue || offs0 == NULL) {
-		_fill_SV_with_vals(vals, offs0, n, buf_sv);
+	if (leaf == R_NilValue) {
+		_write_Rvector_block_to_SV(Rvector, block_offset,
+					   offs0, n, buf_sv);
 	} else {
-		const SparseVec sv1 = leaf2SV(leaf, buf_sv->Rtype,
-					      buf_sv->len,
-					      buf_sv->na_background);
-		_subassign_SV1_with_v2(&sv1, offs0, vals, n, buf_sv);
+		const SparseVec sv = leaf2SV(leaf, buf_sv->Rtype,
+					     buf_sv->len,
+					     buf_sv->na_background);
+		int neffrep;
+		if (offs0 == NULL) {
+			neffrep = _subassign_full_SV_with_Rvector_block(&sv,
+					     Rvector, block_offset, buf_sv);
+		} else {
+			neffrep = _subassign_SV_with_Rvector_block(&sv,
+					     offs0, n,
+					     Rvector, block_offset, buf_sv);
+		}
+		//printf("n = %d / neffrep = %d\n", n, neffrep);
+		if (neffrep == 0)
+			return leaf;  /* no-op */
 	}
 	return SV2leaf(buf_sv);
 }
 
-/****************************************************************************
- * _subassign_leaf_with_Rvector_OLD()
- *
- * TODO: Get rid of this!
- */
-
-
-/* Do NOT use on a NULL leaf. Can be used on a lacunar leaf.
-   'index' must be an integer vector containing valid zero-based indices
-   (a.k.a. offsets) into 'leaf' seen as dense. They are expected to be
-   already sorted in strictly ascending order.
-   'Rvector' must be a vector (atomic or list) of the same length as 'index'.
-   Returns a leaf POSSIBLY CONTAMINATED WITH ZEROS in its 'nzvals'.
-   More precisely: the zeros in 'Rvector' will be injected in the 'nzvals'
-   of the returned leaf. This function does NOT remove them!
-   'nzcount' of the returned leaf is guaranteed to not exceed
-   min(nzcount(leaf) + length(index), INT_MAX).
-   Will NEVER return a NULL. Can ONLY return a lacunar leaf if input leaf
-   is already lacunar **and** 'index' has length 0 (no-op). */
-SEXP _subassign_leaf_with_Rvector_OLD(SEXP leaf, SEXP index, SEXP Rvector)
+/* Can be used on a NULL or lacunar leaf.
+   Both 'offs' and 'selection' must be arrays of 'n' offsets (non-negative
+   integers).
+   The offsets in 'offs' must be strictly sorted (in ascending order) and the
+   last offset in the array must be < 'buf_sv->len'.
+   The offsets in 'selection' must be < 'LENGTH(Rvector)'. They are typically
+   unsorted. */
+SEXP _subassign_leaf_with_Rvector_subset(SEXP leaf, const int *offs, int n,
+		SEXP Rvector, const int *selection, SparseVec *buf_sv)
 {
-	int index_len = LENGTH(index);
-	if (LENGTH(Rvector) != index_len)
-		error("SparseArray internal error in "
-		      "_subassign_leaf_with_Rvector_OLD():\n"
-		      "    'index' and 'Rvector' have different lengths");
-	if (index_len == 0)
-		return leaf;  /* no-op */
-
-	SEXP nzvals, nzoffs;
-	int nzcount = unzip_leaf(leaf, &nzvals, &nzoffs);
-	SEXPTYPE Rtype = TYPEOF(Rvector);
-	if (nzvals != R_NilValue && TYPEOF(nzvals) != Rtype)
-		error("SparseArray internal error in "
-		      "_subassign_leaf_with_Rvector_OLD():\n"
-		      "    'Rvector' and 'leaf' have different types");
-
-	/* Compute 'ans_nzcount'. */
-	const int *nzoffs_p = INTEGER(nzoffs);
-	const int *index_p = INTEGER(index);
-	int ans_nzcount = 0, k1 = 0, k2 = 0;
-	while (k1 < nzcount && k2 < index_len) {
-		if (*nzoffs_p < *index_p) {
-			nzoffs_p++;
-			k1++;
-		} else if (*nzoffs_p > *index_p) {
-			index_p++;
-			k2++;
-		} else {
-			/* *nzoffs_p == *index_p */
-			nzoffs_p++;
-			k1++;
-			index_p++;
-			k2++;
-		}
-		ans_nzcount++;
+	if (leaf == R_NilValue) {
+		_write_Rvector_subset_to_SV(Rvector, selection, offs, n,
+					    buf_sv);
+	} else {
+		const SparseVec sv = leaf2SV(leaf, buf_sv->Rtype,
+					     buf_sv->len,
+					     buf_sv->na_background);
+		int neffrep = _subassign_SV_with_Rvector_subset(&sv,
+					     offs, n,
+					     Rvector, selection, buf_sv);
+		//printf("n = %d / neffrep = %d\n", n, neffrep);
+		if (neffrep == 0)
+			return leaf;  /* no-op */
 	}
-	if (k1 < nzcount) {
-		ans_nzcount += nzcount - k1;
-	} else if (k2 < index_len) {
-		ans_nzcount += index_len - k2;
-	}
+	return SV2leaf(buf_sv);
+}
 
-	CopyRVectorEltFUN copy_Rvector_elt_FUN =
-		_select_copy_Rvector_elt_FUN(Rtype);
-	if (copy_Rvector_elt_FUN == NULL)
-		error("SparseArray internal error in "
-		      "_subassign_leaf_with_Rvector_OLD():\n"
-		      "    type \"%s\" is not supported", type2char(Rtype));
-
-	SEXP ans_nzvals, ans_nzoffs;
-	SEXP ans = PROTECT(_alloc_and_unzip_leaf(Rtype, ans_nzcount,
-						 &ans_nzvals, &ans_nzoffs));
-
-	/* Fill 'ans_nzvals' and 'ans_nzoffs'. */
-	nzoffs_p = INTEGER(nzoffs);
-	index_p = INTEGER(index);
-	int *ans_nzoffs_p = INTEGER(ans_nzoffs);
-	int k = 0;
-	k1 = k2 = 0;
-	while (k1 < nzcount && k2 < index_len) {
-		if (*nzoffs_p < *index_p) {
-			*ans_nzoffs_p = *nzoffs_p;
-			copy_Rvector_elt_FUN(nzvals, (R_xlen_t) k1,
-					     ans_nzvals, (R_xlen_t) k);
-			nzoffs_p++;
-			k1++;
-		} else if (*nzoffs_p > *index_p) {
-			*ans_nzoffs_p = *index_p;
-			copy_Rvector_elt_FUN(Rvector, (R_xlen_t) k2,
-					     ans_nzvals, (R_xlen_t) k);
-			index_p++;
-			k2++;
-		} else {
-			/* *nzoffs_p == *index_p */
-			*ans_nzoffs_p = *index_p;
-			copy_Rvector_elt_FUN(Rvector, (R_xlen_t) k2,
-					     ans_nzvals, (R_xlen_t) k);
-			nzoffs_p++;
-			k1++;
-			index_p++;
-			k2++;
-		}
-		ans_nzoffs_p++;
-		k++;
-	}
-	int n;
-	if (k1 < nzcount) {
-		n = nzcount - k1;
-		memcpy(ans_nzoffs_p, nzoffs_p, sizeof(int) * n);
-		if (nzvals == R_NilValue) {  /* lacunar leaf */
-			_set_Rsubvec_elts_to_one(ans_nzvals, (R_xlen_t) k,
-						 (R_xlen_t) n);
-		} else {  /* standard leaf */
-			_copy_Rvector_elts(nzvals, (R_xlen_t) k1,
-					   ans_nzvals, (R_xlen_t) k,
-					   (R_xlen_t) n);
-		}
-	} else if (k2 < index_len) {
-		n = index_len - k2;
-		memcpy(ans_nzoffs_p, index_p, sizeof(int) * n);
-		_copy_Rvector_elts(Rvector, (R_xlen_t) k2,
-				   ans_nzvals, (R_xlen_t) k,
-				   (R_xlen_t) n);
-	}
-	UNPROTECT(1);
-	return ans;
+/* Can be used on a NULL or lacunar leaf.
+   'offs' and 'xselection' must have length 'n' (they cannot be NULL). */
+SEXP _subassign_leaf_with_Rvector_xsubset(SEXP leaf, const int *offs, int n,
+		SEXP Rvector, const R_xlen_t *xselection, SparseVec *buf_sv)
+{
+	error("_subassign_leaf_with_Rvector_xsubset() is not ready yet");
+	return R_NilValue;
 }
 
