@@ -80,13 +80,48 @@ test_that("subassign an SVT_SparseArray object by an Mindex or Lindex", {
 
 test_that("low-level SparseArray:::.subassign_SVT_by_Nindex()", {
     subassign_SVT_by_Nindex <- SparseArray:::.subassign_SVT_by_Nindex
-    test_subassign_SVT_by_Nindex <- function(svt0, a0, Nindex, value) {
-        svt <- subassign_SVT_by_Nindex(svt0, Nindex, value)
-        storage.mode(a0) <- type(svt)
-        a <- S4Arrays:::subassign_by_Nindex(a0, Nindex, value)
-        expected_class <-
-            if (is.matrix(a)) "SVT_SparseMatrix" else "SVT_SparseArray"
-        check_array_like_object(svt, expected_class, a)
+    test_subassign_SVT_by_Nindex <-
+        function(svt0, Nindex, value, expected_type=type(value)) {
+            svt <- subassign_SVT_by_Nindex(svt0, Nindex, value)
+            expect_identical(type(svt), expected_type)
+            a0 <- as.array(`type<-`(svt0, expected_type))
+            a <- S4Arrays:::subassign_by_Nindex(a0, Nindex, value)
+            expected_class <-
+                if (is.matrix(a)) "SVT_SparseMatrix" else "SVT_SparseArray"
+            check_array_like_object(svt, expected_class, a)
+        }
+
+    ## --- 1D objects ---
+
+    svt0 <- SVT_SparseArray(dim=10, type="raw", dimnames=list(LETTERS[1:10]))
+    a0 <- as.array(svt0)
+    Nindex1 <- list(c(6:9, 2L))
+    Nindex2 <- list(NULL)
+    Nindex3 <- list(c(10L, 3:5, 3L))
+
+    svt1 <- subassign_SVT_by_Nindex(svt0, Nindex1, as.raw(0:4))
+    a1 <- S4Arrays:::subassign_by_Nindex(a0, Nindex1, as.raw(0:4))
+    check_array_like_object(svt1, "SVT_SparseArray", a1)
+
+    for (Nindex in list(Nindex1, Nindex2, Nindex3)) {
+        value <- c(TRUE, FALSE, TRUE, TRUE, FALSE)
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
+        value <- -2:2
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
+        value <- c(-pi, NaN, 0, -Inf, NA)
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
+        value <- 2.44 - value * 8i
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
+        value <- c("hello", "", "world", "", "!")
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
+        value <- list(NULL, 11:14, NULL, factor(), list("a", complex(0), NA))
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
     }
 
     ## --- 2D objects ---
@@ -106,17 +141,23 @@ test_that("low-level SparseArray:::.subassign_SVT_by_Nindex()", {
 
     for (Nindex in list(Nindex1, Nindex2, Nindex3, Nindex4, Nindex5)) {
         value <- c(TRUE, FALSE, TRUE, TRUE, FALSE)
-        test_subassign_SVT_by_Nindex(svt0, m0, Nindex, value)
-        test_subassign_SVT_by_Nindex(svt1, m1, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
         value <- -2:2
-        test_subassign_SVT_by_Nindex(svt0, m0, Nindex, value)
-        test_subassign_SVT_by_Nindex(svt1, m1, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
         value <- c(-pi, NaN, 0, -Inf, NA)
-        test_subassign_SVT_by_Nindex(svt0, m0, Nindex, value)
-        test_subassign_SVT_by_Nindex(svt1, m1, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
         value <- 2.44 - value * 8i
-        test_subassign_SVT_by_Nindex(svt0, m0, Nindex, value)
-        test_subassign_SVT_by_Nindex(svt1, m1, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
+        value <- c("hello", "", "world", "", "!")
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
+        value <- list(NULL, 11:14, NULL, factor(), list("a", complex(0), NA))
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
     }
 
     ## --- 3D objects ---
@@ -136,17 +177,23 @@ test_that("low-level SparseArray:::.subassign_SVT_by_Nindex()", {
 
     for (Nindex in list(Nindex1, Nindex2, Nindex3, Nindex4, Nindex5)) {
         value <- c(TRUE, FALSE, TRUE, TRUE, FALSE)
-        test_subassign_SVT_by_Nindex(svt0, a0, Nindex, value)
-        test_subassign_SVT_by_Nindex(svt1, a1, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
         value <- -2:2
-        test_subassign_SVT_by_Nindex(svt0, a0, Nindex, value)
-        test_subassign_SVT_by_Nindex(svt1, a1, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
         value <- c(-pi, NaN, 0, -Inf, NA)
-        test_subassign_SVT_by_Nindex(svt0, a0, Nindex, value)
-        test_subassign_SVT_by_Nindex(svt1, a1, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
         value <- 2.44 - value * 8i
-        test_subassign_SVT_by_Nindex(svt0, a0, Nindex, value)
-        test_subassign_SVT_by_Nindex(svt1, a1, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
+        value <- c("hello", "", "world", "", "!")
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
+        value <- list(NULL, 11:14, NULL, factor(), list("a", complex(0), NA))
+        test_subassign_SVT_by_Nindex(svt0, Nindex, value)
+        test_subassign_SVT_by_Nindex(svt1, Nindex, value)
     }
 })
 
